use cgmath::{Vector3, Vector4, Zero};
use utilities::{impl_reprc, prelude::*};
use vulkan_rs::prelude::*;

impl_reprc!(
    #[derive(Copy, Debug)]
    pub struct PositionNormalColor {
        #[assume_reprc]
        pub position: Vector3<f32>,

        #[assume_reprc]
        filler_p: f32,

        #[assume_reprc]
        pub normal: Vector3<f32>,

        #[assume_reprc]
        filler_n: f32,

        #[assume_reprc]
        pub color: Vector4<f32>,
    }
);

impl PositionNormalColor {
    pub fn new(position: Vector3<f32>, normal: Vector3<f32>, color: Vector4<f32>) -> Self {
        Self {
            position,
            filler_p: 1.0,
            normal,
            filler_n: 0.0,
            color,
        }
    }
}

impl Default for PositionNormalColor {
    fn default() -> Self {
        PositionNormalColor {
            position: Vector3::zero(),
            filler_p: 1.0,
            normal: Vector3::zero(),
            filler_n: 0.0,
            color: Vector4::zero(),
        }
    }
}

impl VertexInputDescription for PositionNormalColor {
    fn attributes() -> Vec<VkVertexInputAttributeDescription> {
        vec![
            // position
            VkVertexInputAttributeDescription {
                location: 0,
                binding: 0,
                format: VK_FORMAT_R32G32B32_SFLOAT,
                offset: 0,
            },
            // normal
            VkVertexInputAttributeDescription {
                location: 1,
                binding: 0,
                format: VK_FORMAT_R32G32B32_SFLOAT,
                offset: 16,
            },
            // color
            VkVertexInputAttributeDescription {
                location: 3,
                binding: 0,
                format: VK_FORMAT_R32G32B32A32_SFLOAT,
                offset: 32,
            },
        ]
    }
}
